/***************************************************************************
Name: CookieSafe
Description: Control cookie permissions.
Author: Ron Beckman
Homepage: http://addons.mozilla.org

Copyright (C) 2007  Ron Beckman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

Free Software Foundation, Inc.
51 Franklin Street
Fifth Floor
Boston, MA  02110-1301
USA
***************************************************************************/



window.addEventListener('load', cookiesafeInit, false);

function cookiesafeInit() {
	//make sure the windowtype attribute is set in TB
	var brows = cookiesafe.getAppInfo();
	if (brows.name=='Thunderbird') {
		var win = document.getElementById('messengerWindow');
		var type = win.getAttribute('windowtype');
		if (!type) win.setAttribute('windowtype','mail:3pane');
	}
	cookiesafe.init();
}

window.addEventListener('unload', cookiesafeExit, false);

function cookiesafeExit() {
	cookiesafe.exit();
}

/*changeTabsSM() is used to display the exceptions tab in SeaMonkey
 and Mozilla browsers. SM and Moz do not use separate windows to
 display cookies and exceptions. therefore it is necessary to switch
 tabs in order to display the exceptions since both features appear in
 the same window.*/

function changeTabSM() {
	var win = cookiesafe.getWin('mozilla:cookieviewer');
	var tab = win.document.getElementById('permissionsTab');
	win.document.getElementById('tabbox').selectedTab=tab;
}

/*cookiesafeResetIcon() is used to ensure that the cookiesafe icon is
 always reset to the correct image after a cookie notification has
 occurred. instead of resetting the icon after a page load or location
 change, a timeout is used to call the load method.*/

function cookiesafeResetIcon() {
	cookiesafe.load();
}

/*cookiesafeInitPermObserver() is used to prevent an overflow of perm-changed
 notifications from being sent to the perm observer in rapid succession. that
 can occur by clearing all of the exceptions, by deleting multiple cookie
 exceptions, or by updating the blocklist which may add hundreds of exceptions.*/

function cookiesafeInitPermObserver() {
	csPermObserver.init();

	/*the following code is needed since by disabling the perm observer some
	exceptions may not be caught that appear in the last 10 hosts list or the
	current webpage's domain*/

	if (!cookiesafe.isFlashing()) cookiesafe.load();
	var os = Components.classes["@mozilla.org/observer-service;1"].
		getService(Components.interfaces.nsIObserverService);
	os.notifyObservers(null,'cookiesafe-hosts-changed',null);
}

var cookiesafeProgress = {
	QueryInterface: function(aIID) {
		if (aIID.equals(Components.interfaces.nsIWebProgressListener)   ||
		    aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
		    aIID.equals(Components.interfaces.nsISupports))
      			return this;
		throw Components.results.NS_NOINTERFACE;
	},
	onLinkIconAvailable: function(a) { },
	onLocationChange: function(a,b,c) {
		if (!cookiesafe.isFlashing()) cookiesafe.load();
	},
	onProgressChange: function(a,b,c,d,e,f) { },
	onSecurityChange: function(a,b,c) { },
	onStateChange: function(a,b,c,d) { },
	onStatusChange: function(a,b,c,d) { }
};

var cookiesafe = {

	TB2: false,

	_block   : "",
	_remove  : "",
	_allow   : "",
	_session : [],
	_temp    : [],
	_clear   : [],
	_export  : [],
	_order   : 0,

	getURI: function(url) {
		return Components.classes["@mozilla.org/network/io-service;1"].
		getService(Components.interfaces.nsIIOService).
		newURI(url,null,null);
	},

	isFlashing: function() {
		var dom = this.getDOM();
		var flashing = dom.sts.getAttribute('cookiesafe');
		return (flashing.substr(0,5)=='flash') ? true : false;
	},

	getObserver: function() {
		return Components.classes["@mozilla.org/observer-service;1"].
		getService(Components.interfaces.nsIObserverService);
	},

	getCS: function() {
		return Components.classes['@mozilla.org/CookieSafe;1'].
		createInstance(Components.interfaces.nsICookieSafe);
	},

	getCSHiddenMenuItems: function() {
		return Components.classes['@mozilla.org/CSHiddenMenuItems;1'].
		createInstance(Components.interfaces.nsICSHiddenMenuItems);
	},

	getCSLast10Hosts: function() {
		return Components.classes['@mozilla.org/CSLast10Hosts;1'].
		getService(Components.interfaces.nsICSLast10Hosts);
	},

	getCSTempExceptions: function() {
		return Components.classes['@mozilla.org/CSTempExceptions;1'].
		getService(Components.interfaces.nsICSTempExceptions);
	},

	getBaseDomain: function(host) {
		var cs = this.getCS();
		return cs.removeSub(host);
	},

	getDomain: function() {
		var site = null;
		try {
			site = window.content.document.domain;
			if (!site) {
				var url = window.content.location.href;
				if (url.substr(0,5)=='file:') site = 'scheme:file';
			}
		} catch(e) {
			try {
				var url = window.content.location.href;
				if (url.substr(0,5)=='file:') site = 'scheme:file';
			} catch(e) { }
		}
		return site;
	},

	testPermission: function(host) {
		var url = (host=='scheme:file') ? 'file:///cookiesafe' : 'http://'+host;
		var uri = this.getURI(url);
		var mngr = this.getPermManager();
		var action = mngr.testPermission(uri,'cookie');
		return action;
	},

	getPrefs: function() {
		return Components.classes["@mozilla.org/preferences-service;1"].
		getService(Components.interfaces.nsIPrefService).
		getBranch("cookiesafe.");
	},

	getGlobalPrefs: function() {
		return Components.classes["@mozilla.org/preferences-service;1"].
		getService(Components.interfaces.nsIPrefService).
		getBranch("network.cookie.");
	},

	getWin: function(name) {
		return Components.classes["@mozilla.org/appshell/window-mediator;1"].
		getService(Components.interfaces.nsIWindowMediator).
		getMostRecentWindow(name);
	},

	getAppInfo: function() {
		return Components.classes['@mozilla.org/xre/app-info;1'].
		createInstance(Components.interfaces.nsIXULAppInfo);
	},

	getStr: function(name,bundle) {
		return document.getElementById((bundle) ? bundle : 'cookiesafe-bundle').getString(name);
	},

	getProfile: function() {
		return Components.classes["@mozilla.org/file/directory_service;1"].
		getService(Components.interfaces.nsIProperties).
		get("ProfD", Components.interfaces.nsIFile);
	},

	getPermManager: function() {
		//make sure the browser includes the nsipermissionmanager interface
		if ('nsIPermissionManager' in Components.interfaces) {
			return Components.classes["@mozilla.org/permissionmanager;1"].
			getService(Components.interfaces.nsIPermissionManager);
		} else {
			return Components.classes["@mozilla.org/CSPermManager;1"].
			getService(Components.interfaces.nsICSPermManager);
		}
	},

	getCookieManager: function() {
		return Components.classes["@mozilla.org/cookiemanager;1"].
		getService(Components.interfaces.nsICookieManager);
	},

	getCookieManager2: function() {
		return Components.classes["@mozilla.org/cookiemanager;1"].
		getService(Components.interfaces.nsICookieManager2);
	},

	getExtManager: function() {
		var mngr = Components.classes['@mozilla.org/extensions/manager;1'];
		return (mngr) ? mngr.getService(Components.interfaces.nsIExtensionManager) : false;
	},

	restartBrowser: function() {
		var nsIAppStartup = Components.interfaces.nsIAppStartup;
		Components.classes["@mozilla.org/toolkit/app-startup;1"].
		getService(nsIAppStartup).
		quit(nsIAppStartup.eForceQuit | nsIAppStartup.eRestart);
	},

	showDBWarning: function(type,path) {
		var dom = this.getDOM();
		var img = 'chrome://cookiesafe/skin/warning.gif';
		var msg = (type=='notfound') ? this.getStr('cookiesafe.DBNotFound') : this.getStr('cookiesafe.DBReadOnly');

		if (dom.sts) dom.sts.setAttribute('src',img);
		if (dom.btn) dom.btn.setAttribute('image',img);
		if (dom.ctx) dom.ctx.setAttribute('image',img);

		alert(msg+'\n\n'+path);
	},

	getDOM: function() {
		var sts = document.getElementById('cookiesafe-status');
		var ctx = document.getElementById('cookiesafe-context');
		var ctx2 = document.getElementById('cookiesafe-context2');
		var btn = document.getElementById('cookiesafe-button');
		var CACM = document.getElementById('contentAreaContextMenu');

		return {sts:sts,ctx:ctx,ctx2:ctx2,btn:btn,CACM:CACM};
	},

	getWindowTypeForApp: function(name) {
		switch (name) {
			case 'Thunderbird':
				return 'mail:3pane';

			case 'Songbird':
				return 'Songbird:Main';

			case 'eMusic Remote':
				return 'emusic:window';

			default:
				return 'navigator:browser';
		}
	},

	countOpenWindows: function() {
		var brows = this.getAppInfo();
		var wins = Components.classes["@mozilla.org/appshell/window-mediator;1"].
		getService(Components.interfaces.nsIWindowMediator).
		getEnumerator(this.getWindowTypeForApp(brows.name));

		var win,count=0;
		while (('hasMoreElements' in wins && wins.hasMoreElements()) ||
			 ('hasMore' in wins && wins.hasMore())) {
			win = wins.getNext();
			++count;
		}
		return count;
	},

	hideStatus: function() {
		var dom = this.getDOM();
		var prefs = this.getPrefs();
		dom.sts.setAttribute('hidden',prefs.getBoolPref('hideStatus'));
	},

	hideContext: function() {
		var dom = this.getDOM();
		if (!dom.ctx || !dom.CACM) return false;

		var prefs = this.getPrefs();
		var hc = prefs.getBoolPref('hideContext');
		if (dom.ctx) dom.ctx.setAttribute('hidden',hc);

		if (hc) {
			dom.CACM.removeEventListener('popupshowing', cookiesafe.showContextPopup, false);
		} else {
			dom.CACM.addEventListener('popupshowing', cookiesafe.showContextPopup, false);
		}
		return false;
	},

	init: function() {
		/*we have to place the startup code in this overlay since some of the required
		interfaces and components are not available when the app-startup and xpcom-startup
		notifications are sent to the observer. so we count how many browser windows are
		open and call cookiesafeStartup if this is the first window opened*/

		var num = this.countOpenWindows();
		if (num <= 1) cookiesafeStartup.init();

		//register cookiesafe observers
		csPermObserver.init();
		csCookieObserver.init();
		csPrefObserver.init();

		//add progress listener for all location changes
		getBrowser().addProgressListener(cookiesafeProgress);

		//show-hide statusbar icon
		this.hideStatus();

		//show-hide context menu item
		this.hideContext();

		//save strings and string order for left click popup menu
		this._block = this.getStr('cookiesafe.BlockMI');
		this._remove = this.getStr('cookiesafe.RemoveMI');
		this._allow = this.getStr('cookiesafe.AllowMI');
		this._session = [this.getStr('cookiesafe.Session1MI'),this.getStr('cookiesafe.Session2MI')];
		this._temp = [this.getStr('cookiesafe.Temp1MI'),this.getStr('cookiesafe.Temp2MI')];
		this._clear = [this.getStr('cookiesafe.ClearCookies1MI'),this.getStr('cookiesafe.ClearCookies2MI')];
		this._export = [this.getStr('cookiesafe.ExportCookies1MI'),this.getStr('cookiesafe.ExportCookies2MI')];
		this._order = parseInt(this.getStr('cookiesafe.MenuitemStringOrder','cookiesafe-menuitems-bundle'));

		this.load();
	},

	exit: function() {
		//unregister observers
		csPermObserver.uninit();
		csCookieObserver.uninit();
		csPrefObserver.uninit();

		//remove progress listener
		getBrowser().removeProgressListener(cookiesafeProgress);
	},

	getHostActionImg: function() {
		var actionBase,actionFull,img,site;
		var base = null;
		var prefs = this.getPrefs();
		var host = this.getDomain();

		//change toolbar and statusbar icons
		if (host) {
			base = this.getBaseDomain(host);
			actionBase = this.testPermission(base);
			actionFull = this.testPermission(host);

			/*if the base domain has the same permission as the full domain then you
			know the base domain is saved in the exceptions. there's an outside chance
			that both domains could appear in the exceptions with the same permission
			but it's a redundant operation so we don't test for it. if the base domain
			permission is different from the full domain then either the full domain
			appears in the exceptions or both the full and base domains have different
			permissions assigned to them. in either case we will display the full domain
			to guarantee that the correct icon and menu item labels are displayed.*/

			if (actionFull==8) {
				var tempExc = this.getCSTempExceptions();
				if (tempExc.testTempExceptions((actionFull == actionBase) ? base : host)) {
					img = 'temp';
				} else {
					img = 'session';
				}
			} else if (actionFull==1) {
				img = 'allowed';
			} else if (actionFull==2) {
				img = 'blocked';
			}
		}

		//if no exception is saved change icons based on global prefs
		if (!actionFull) {
			var gPrefs = this.getGlobalPrefs();
			var behavior = gPrefs.getIntPref('cookieBehavior');
			var policy = gPrefs.getIntPref('lifetimePolicy');
			if (behavior==2) {
				img = 'blocked';
			} else {
				img = (policy==2) ? 'session' : 'allowed';
			}
		}

		//determine whether to display full or base domain on menu items (see note above)
		if (actionFull == actionBase) {
			if (actionBase == 0) {
				//display based on user pref
				site = (prefs.getBoolPref('displayBaseDomain')) ? base : host;
			} else {
				//display base
				site = base;
			}
		} else {
			//display full
			site = host;
		}

		//don't change the action variables until after the correct host to display
		//on the menu items has been determined. otherwise the temporary menu item
		//may display the base domain even if a full domain exception has been saved.
		if (img == 'temp') {
			actionFull = actionBase = 'T';
		}

		return {action:actionFull,host:site,image:img};
	},

	load: function() {
		var info = this.getHostActionImg();

		//set cookiesafe icons
		var dom = this.getDOM();
		if (dom.sts) dom.sts.setAttribute('cookiesafe',info.image);
		if (dom.btn) dom.btn.setAttribute('cookiesafe',info.image);
		if (dom.ctx) dom.ctx.setAttribute('cookiesafe',info.image);
	},

	populateMenu: function(item) {
		var elem;
		var mitems = [];
		var elmnts = [];
		var dom = this.getDOM();
		var info = this.getHostActionImg();
		var hmi = this.getCSHiddenMenuItems();
		var hosts = hmi.getHiddenMenuItems();

		while(item.firstChild &&
		      item.firstChild.id != 'cookiesafe-context2') {
		      item.removeChild(item.firstChild);
		}

		if (item.id=='cookiesafe-status-menupopup') {
			mitems = mitems.concat(cookiesafeMenuItems.leftclick);
		} else if (item.id=='cookiesafe-button-menupopup') {
			mitems = mitems.concat(cookiesafeMenuItems.leftclick);
			mitems.reverse();
		} else if (item.id=='cookiesafe-context-menupopup') {
			mitems = mitems.concat(
					cookiesafeMenuItems.leftclick.concat(
					cookiesafeMenuItems.contextOnly));
			dom.ctx2.setAttribute('hidden',(hosts.indexOf('cookiesafe-context2')==-1) ? false : true);
		} else {
			mitems = mitems.concat(cookiesafeMenuItems.rightclick);
		}

		for (var i=0; i<mitems.length; ++i) {
			if (hosts.indexOf(mitems[i][1][1])==-1) {
				elem = this.createMenuItem(mitems[i]);
				if (item.id.search(/^cookiesafe\-menupopup/) == -1 ||
				    elem.id == 'cookiesafe-clear-domain-cookies' ||
				    elem.id == 'cookiesafe-export-domain-cookies') {
					elem = this.changeAttributes(elem,info);
				}
				elmnts.push(elem);
			}
		}

		for (var i=0; i<elmnts.length; ++i) {
			if (item.id == 'cookiesafe-context-menupopup') {
				item.insertBefore(elmnts[i],dom.ctx2);
			} else {
				item.appendChild(elmnts[i]);
			}
		}
	},

	createMenuItem: function(item) {
		var mnuitm = document.createElement(item[0]);
		for (var i=1; i<item.length; ++i) {
			if (item[i][0]=='label') {
				mnuitm.setAttribute(item[i][0],this.getStr(item[i][1]));
			} else {
				mnuitm.setAttribute(item[i][0],item[i][1]);
			}
		}
		return mnuitm;
	},

	changeAttributes: function(item,info) {
		//the order of the strings and hosts on the menu item labels need
		//to be reversed for some locales, the order is saved as the
		//cookiesafe.MenuitemStringOrder string in each locale, 0 = string
		//then host, 1 = host then string, 2 = host then string and reverse
		//the strings, we use the _order variable for these purposes

		var label,base;
		var gPrefs = this.getGlobalPrefs();
		var behavior = gPrefs.getIntPref('cookieBehavior');

		//set attribute to either hidden or disabled based on user pref
		var prefs = cookiesafe.getPrefs();
		var hide = prefs.getBoolPref('hideMenuItem');
		var attr = (hide) ? 'hidden' : 'disabled';

		//use base domain for clear and export domain cookies
		if (item.id == 'cookiesafe-clear-domain-cookies' ||
		    item.id == 'cookiesafe-export-domain-cookies') {
			info.host = this.getBaseDomain(info.host);
		}

		//change display name for file urls
		var name = (info.host=='scheme:file') ? 'file://' : info.host;

		switch (item.id) {

			case 'cookiesafe-global-allow':
				item.setAttribute('hidden',(behavior!=2) ? true : false);
				break;

			case 'cookiesafe-global-deny':
				item.setAttribute('hidden',(behavior==2) ? true : false);
				break;

			case 'cookiesafe-sep1':
				item.setAttribute('hidden',(!info.host) ? true : false);
				break;

			case 'cookiesafe-block':
				item.setAttribute((!info.host) ? 'hidden' : attr,(!info.host || info.action==2) ? true : false);
				item.setAttribute('label',(this._order < 1) ? this._block+" "+name : name+" "+this._block);
				item.setAttribute('oncommand','cookiesafe.addException("'+info.host+'","block");');
				break;

			case 'cookiesafe-remove':
				item.setAttribute('hidden',(!info.host || !info.action) ? true : false);
				item.setAttribute('label',(this._order < 1) ? this._remove+" "+name : name+" "+this._remove);
				item.setAttribute('oncommand','cookiesafe.addException("'+info.host+'","remove");');
				break;

			case 'cookiesafe-sep2':
				item.setAttribute('hidden',(!info.host) ? true : false);
				break;

			case 'cookiesafe-allow':
				item.setAttribute((!info.host) ? 'hidden' : attr,(!info.host || info.action==1) ? true : false);
				item.setAttribute('label',(this._order < 1) ? this._allow+" "+name : name+" "+this._allow);
				item.setAttribute('oncommand','cookiesafe.addException("'+info.host+'","allow");');
				break;

			case 'cookiesafe-session':
				if (this._order == 0) {
					label = this._session[0]+" "+name+" "+this._session[1];
				} else if (this._order == 1) {
					label = name+" "+this._session[0]+" "+this._session[1];
				} else {
					label = name+" "+this._session[1]+" "+this._session[0];
				}
				item.setAttribute('label',label);
				item.setAttribute((!info.host) ? 'hidden' : attr,(!info.host || info.action==8) ? true : false);
				item.setAttribute('oncommand','cookiesafe.addException("'+info.host+'","session");');
				break;

			case 'cookiesafe-temporary':
				if (this._order == 0) {
					label = this._temp[0]+" "+name+" "+this._temp[1];
				} else if (this._order == 1) {
					label = name+" "+this._temp[0]+" "+this._temp[1];
				} else {
					label = name+" "+this._temp[1]+" "+this._temp[0];
				}
				item.setAttribute('label',label);
				item.setAttribute((!info.host) ? 'hidden' : attr,(!info.host || info.action=='T') ? true : false);
				item.setAttribute('oncommand','cookiesafe.addException("'+info.host+'","temp");');
				break;

			case 'cookiesafe-clear-domain-cookies':
				if (this._order == 0) {
					label = this._clear[0]+" "+name+" "+this._clear[1];
				} else if (this._order == 1) {
					label = name+" "+this._clear[0]+" "+this._clear[1];
				} else {
					label = name+" "+this._clear[1]+" "+this._clear[0];
				}
				item.setAttribute('label',label);
				item.setAttribute('hidden',(!info.host) ? true : false);
				item.setAttribute('oncommand','cookiesafe.clearDomainCookies("'+info.host+'");');
				break;

			case 'cookiesafe-export-domain-cookies':
				if (this._order == 0) {
					label = this._export[0]+" "+name+" "+this._export[1];
				} else if (this._order == 1) {
					label = name+" "+this._export[0]+" "+this._export[1];
				} else {
					label = name+" "+this._export[1]+" "+this._export[0];
				}
				item.setAttribute('label',label);
				item.setAttribute('hidden',(!info.host) ? true : false);
				item.setAttribute('oncommand','cookiesafe.exportDomainCookies("'+info.host+'");');
				break;
		}

		return item;
	},

	createTooltip: function(popup,event) {
		var ck,baseck,hbox,label1,label2,label3,label4;

		while (popup.firstChild) {
			popup.removeChild(popup.firstChild);
		}

		var prefs = this.getPrefs();
		if (prefs.getBoolPref('hideTooltip')) {
			event.preventDefault();
			popup.hidePopup();
			return false;
		}

		//get the base domain for the current page
		var basepg = this.getBaseDomain(this.getDomain());

		//get label strings
		var cln = this.getStr('cookiesafe.Colon');
		var hst = this.getStr('cookiesafe.lHost');
		var nme = this.getStr('cookiesafe.lName');

		//loop through cookies and export them one at a time
		var mngr = this.getCookieManager();
		var cookies = mngr.enumerator;
		while (cookies.hasMoreElements()) {
			ck = cookies.getNext();
			ck.QueryInterface(Components.interfaces.nsICookie);
			ck.QueryInterface(Components.interfaces.nsICookie2);

			//make sure cookie relates to the current page
			baseck = this.getBaseDomain(ck.rawHost);
			if (baseck != basepg) continue;

			hbox = document.createElement('hbox');

			label1 = document.createElement('label');
			label1.setAttribute('value',hst+cln);
			label1.setAttribute('class','boldNormal');
			label2 = document.createElement('label');
			label2.setAttribute('value',ck.host);
			label2.setAttribute('class','tooltipWidth');
			label2.setAttribute('crop','right');
			label3 = document.createElement('label');
			label3.setAttribute('value',nme+cln);
			label3.setAttribute('class','boldNormal');
			label4 = document.createElement('label');
			label4.setAttribute('value',ck.name);
			label4.setAttribute('class','tooltipWidth');
			label4.setAttribute('crop','right');

			hbox.appendChild(label1);
			hbox.appendChild(label2);
			hbox.appendChild(label3);
			hbox.appendChild(label4);
			popup.appendChild(hbox);
		}

		if (popup.childNodes.length == 0) {
			event.preventDefault();
			popup.hidePopup();
		}

		return false;
	},

	allowAll: function(allow,bhvr) {
		var prefs = this.getPrefs();
		var gPrefs = this.getGlobalPrefs();

		/*when autoRefresh is called while the cookieBehavior observer is active
		it causes a bug which prevents the user from typing in the location bar.
		it appears that focus gets stuck on the popup menu. pressing A will allow
		cookies globally while pressing D will deny cookies globally. until this
		bug can be resolved the autoRefresh method must be called from a timeout to
		allow the pref observer enough time to call the load() method.*/

		if (allow) {
			gPrefs.setIntPref('cookieBehavior',bhvr);
			window.setTimeout(cookiesafe.autoRefresh,500);
		} else {
			gPrefs.setIntPref('cookieBehavior',2);
		}
	},

	isValidDomain: function(host) {
		var test = host.search(/[~!@#$%^&*()_+|`=\\\[\];',\/{}"<>? ]/);
		return (test==-1) ? true : false;
	},

	addException: function(site,exc) {
		if (!site || !exc) return false;

		var prefs = this.getPrefs();
		var prDom = prefs.getBoolPref('promptDomain');

		if (prDom) {
			site = prompt(this.getStr('cookiesafe.EnterDomain'),site);
			if (!site) return false;
			while (!this.isValidDomain(site)) {
				site = prompt(this.getStr('cookiesafe.DomainError'),site);
				if (!site) return false;
			}
		}

		this.addException2(site,exc);
		return false;
	},

	addException2: function(site,exc) {
		if (!site || !exc) return false;

		//remove temp exception if it exists
		var tempExc = this.getCSTempExceptions();
		tempExc.removeTempExceptions(site);

		var mngr = this.getPermManager();
		var url = (site=='scheme:file') ? 'file:///cookiesafe' : 'http://'+site;
		var uri = this.getURI(url);
		var type = (exc=='remove') ? 'deleted' : 'added';

		if (exc=='remove') {
			mngr.remove(site,'cookie');
		} else if (exc=='allow') {
			mngr.add(uri,'cookie',1);
		} else if (exc=='block') {
			mngr.add(uri,'cookie',2);
		} else if (exc=='session') {
			mngr.add(uri,'cookie',8);
			if (!cookiesafe.isFlashing()) cookiesafe.load();
		} else if (exc=='temp') {
			tempExc.addTempExceptions(site);
			mngr.add(uri,'cookie',8);
			if (!cookiesafe.isFlashing()) cookiesafe.load();
		}

		//notify csperm-changed observer in TB2 only
		if (this.TB2) {
			var os = this.getObserver();
			os.notifyObservers(null,'csperm-changed',type);
		}

		window.setTimeout(cookiesafe.autoRefresh,500);
		return false;
	},

	autoRefresh: function() {
		var prefs = cookiesafe.getPrefs();
		var auto = prefs.getBoolPref('autoRefresh');
		try {
			if (auto) window.content.location.reload();
		} catch(e) { }
	},

	showContextPopup: function() {
		if ('gContextMenu' in window && gContextMenu) {
			var dom = cookiesafe.getDOM();
			var show = gContextMenu.inDirList ||
					gContextMenu.onImage ||
					gContextMenu.isContentSelected ||
					gContextMenu.onLink ||
					gContextMenu.onTextInput;
			if (dom.ctx) dom.ctx.setAttribute('hidden',(show) ? true : false);
		}
	},

	hideMenuItem: function(item) {
		if (!item) return false;

		//remove oncommand attribute before proceeding, in gecko 1.9 the
		//ondraggesture event does not automatically cancel other events
		//on the same element when it's triggered which can cause both
		//ondraggesture and oncommand to be triggered at the same time
		item.removeAttribute('oncommand');

		var prefs = this.getPrefs();
		var ask = prefs.getBoolPref('promptMenuItem');
		var rem = (ask) ? confirm(this.getStr('cookiesafe.ConfirmHideItem')) : true;
		if (rem) this.hideMenuItem2(item);
		return false;
	},

	hideMenuItem2: function(item) {
		if (!item) return false;

		var hmi = this.getCSHiddenMenuItems();
		hmi.addHiddenMenuItems(item.id);

		//make sure options window is updated if open
		var os = this.getObserver();
		os.notifyObservers(item,'cookiesafe-menuitems-changed','deleted');
		return false;
	},

	removeTempExceptions: function() {
		var prefs = this.getPrefs();
		var perms = prefs.getCharPref('tempExceptions');
		if (!perms) return false;

		//remove temp exceptions
		perms = perms.split(' ');
		var mngr = this.getPermManager();
		for (var i=0; i<perms.length; ++i) {
			if (!perms[i]) continue;
			try {
				mngr.remove(perms[i],'cookie');
			} catch(e) {
				continue;
			}
		}

		prefs.setCharPref('tempExceptions','');
		return false;
	},

	clearCookies: function() {
		var prefs = this.getPrefs();
		var ask = prefs.getBoolPref('promptClear');
		var clr = (ask) ? confirm(this.getStr('cookiesafe.ConfirmClearCk')) : true;
		if (clr) this.clearCookies2();
	},

	clearCookies2: function() {
		var mngr = this.getCookieManager();
		mngr.removeAll();
	},

	clearExceptions: function() {
		var prefs = this.getPrefs();
		var ask = prefs.getBoolPref('promptClear');
		var clr = (ask) ? confirm(this.getStr('cookiesafe.ConfirmClearEx')) : true;
		if (clr) this.clearExceptions2();
	},

	clearExceptions2: function() {
		var prefs = this.getPrefs();
		var protectBlocked = prefs.getBoolPref('protectBlockedSites');
		var tempExc = this.getCSTempExceptions();
		tempExc.clearTempExceptions();

		var exc,perms,temp;
		var mngr = this.getPermManager();
		if (mngr instanceof Components.interfaces.nsIPermissionManager) {
			perms = mngr.enumerator;
			while (('hasMoreElements' in perms && perms.hasMoreElements()) ||
				 ('hasMore' in perms && perms.hasMore())) {
				exc = perms.getNext();
				exc.QueryInterface(Components.interfaces.nsIPermission);
				if (protectBlocked && exc.capability == 2) continue;
				if (exc.type=='cookie') {
					mngr.remove(exc.host,'cookie');
				}
			}
		} else { //for TB2 only
			if (protectBlocked) {
				var perms = mngr.getAllPermissions().split(' ');
				for (var i=0; i<perms.length; ++i) {
					if (!perms[i]) continue;
					temp = perms[i].split('|');
					temp[1] = parseInt(temp[1]);
					if (temp[1] == 2) {
						continue;
					} else {
						mngr.remove(temp[0],'cookie');
					}
				}
			} else {
				mngr.removeAll();
			}
			var os = this.getObserver();
			os.notifyObservers(null,'csperm-changed','cleared');
		}
	},

	processCookies: function(req) {
		if (req.readyState == 4) {
			/*for some reason if the user is not connected to the network the status
			property will throw an error when it's accessed. Although when you query
			it using ('status' in req) it returns true? At this point the only thing
			to do is enclose the req.status command in try-catch brackets.*/

			try { var status = req.status; } catch(e) { return false; }

			if (status == 200 || status == 0) {
				var dt = new Date();
				var time = parseInt(dt.getTime() / 1000);
				var mngr = this.getCookieManager2();
				var doc = req.responseXML;
				var cookies = doc.getElementsByTagName('cookie');
				var hostTag,host,pathTag,path,nameTag,name,valueTag,value;
				var secureTag,secure,httpTag,http,sessionTag,session,expiresTag,expires;

				for (var i=0; i<cookies.length; ++i) {
					hostTag = cookies[i].getElementsByTagName('host');
					if (hostTag[0].getAttribute('escaped') == 'true') {
						host = unescape(hostTag[0].textContent);
					} else {
						host = hostTag[0].textContent;
					}
					pathTag = cookies[i].getElementsByTagName('path');
					if (pathTag[0].getAttribute('escaped') == 'true') {
						path = unescape(pathTag[0].textContent);
					} else {
						path = pathTag[0].textContent;
					}
					nameTag = cookies[i].getElementsByTagName('name');
					if (nameTag[0].getAttribute('escaped') == 'true') {
						name = unescape(nameTag[0].textContent);
					} else {
						name = nameTag[0].textContent;
					}
					valueTag = cookies[i].getElementsByTagName('value');
					if (valueTag[0].getAttribute('escaped') == 'true') {
						value = unescape(valueTag[0].textContent);
					} else {
						value = valueTag[0].textContent;
					}
					secureTag = cookies[i].getElementsByTagName('secure');
					secure = (secureTag[0].textContent == 'true') ? true : false;
					httpTag = cookies[i].getElementsByTagName('http');
					http = (httpTag[0].textContent == 'true') ? true : false;
					sessionTag = cookies[i].getElementsByTagName('session');
					session = (sessionTag[0].textContent == 'true') ? true : false;
					expiresTag = cookies[i].getElementsByTagName('expires');
					expires = parseInt(expiresTag[0].textContent);
					if (session == true || expires == 0) {
						session = true;
						expires = time + 86400;
					}

					if ('cookieExists' in mngr) {
						mngr.add(host,path,name,value,secure,http,session,expires);
					} else {
						mngr.add(host,path,name,value,secure,session,expires);
					}
				}
			}
		}
		return false;
	},

	processExceptions: function(req) {
		if (req.readyState == 4) {
			/*for some reason if the user is not connected to the network the status
			property will throw an error when it's accessed. Although when you query
			it using ('status' in req) it returns true? At this point the only thing
			to do is enclose the req.status command in try-catch brackets.*/

			try { var status = req.status; } catch(e) { return false; }

			if (status == 200 || status == 0) {
				var mngr = this.getPermManager();
				var doc = req.responseXML;
				var exceptions = doc.getElementsByTagName('exception');
				var hostTag,host,capabilityTag,cap,url,uri;

				for (var i=0; i<exceptions.length; ++i) {
					hostTag = exceptions[i].getElementsByTagName('host');
					if (hostTag[0].getAttribute('escaped') == 'true') {
						host = unescape(hostTag[0].textContent);
					} else {
						host = hostTag[0].textContent;
					}

					capabilityTag = exceptions[i].getElementsByTagName('capability');
					cap = parseInt(capabilityTag[0].textContent);

					url = (host=='scheme:file') ? 'file:///cookiesafe' : 'http://'+host;
					uri = this.getURI(url);
					mngr.add(uri,'cookie',cap);
				}
			}
		}
		return false;
	},

	importCookies: function(loading) {
		var prefs = this.getPrefs();
		var prloc = prefs.getBoolPref('promptImpExpLocation');
		var url = prefs.getCharPref('cookiesLocation');

		//make sure a url is being used and not a file path.
		//versions of CS prior to 3.0 used file paths
		try {
			var uri = this.getURI(url);
			if (uri.scheme.search(/^file|^http|^ftp/) == -1) {
				url = '';
				prefs.setCharPref('cookiesLocation',url);
			}
		} catch(e) {
			url = '';
			prefs.setCharPref('cookiesLocation',url);
		}

		if (loading) {
			if (!url) return false;
		} else if (!url || prloc) {
			var file = this.getFilePicker('open','cookies.xml');
			if (!file) return false;
			url = this.convertFileToUrl(file);
		}

		try {
			var httpRequest = new XMLHttpRequest();
			httpRequest.overrideMimeType('text/xml');
			httpRequest.onreadystatechange = function() { cookiesafe.processCookies(httpRequest); };
			httpRequest.open('GET', url, true);
			httpRequest.send(null);
		} catch(e) {
			return false;
		}

		return false;
	},

	importExceptions: function(loading) {
		var prefs = this.getPrefs();
		var prloc = prefs.getBoolPref('promptImpExpLocation');
		var url = prefs.getCharPref('exceptionsLocation');

		//make sure a url is being used and not a file path.
		//versions of CS prior to 3.0 used file paths
		try {
			var uri = this.getURI(url);
			if (uri.scheme.search(/^file|^http|^ftp/) == -1) {
				url = '';
				prefs.setCharPref('exceptionsLocation',url);
			}
		} catch(e) {
			url = '';
			prefs.setCharPref('exceptionsLocation',url);
		}

		if (loading) {
			if (!url) return false;
		} else if (!url || prloc) {
			var file = this.getFilePicker('open','exceptions.xml');
			if (!file) return false;
			url = this.convertFileToUrl(file);
		}

		try {
			var httpRequest = new XMLHttpRequest();
			httpRequest.overrideMimeType('text/xml');
			httpRequest.onreadystatechange = function() { cookiesafe.processExceptions(httpRequest); };
			httpRequest.open('GET', url, true);
			httpRequest.send(null);
		} catch(e) {
			return false;
		}

		return false;
	},

	convertFileToUrl: function(file) {
		return Components.classes['@mozilla.org/network/protocol;1?name=file'].
		createInstance(Components.interfaces.nsIFileProtocolHandler).
		getURLSpecFromFile(file);
	},

	convertUrlToFile: function(url) {
		return Components.classes['@mozilla.org/network/protocol;1?name=file'].
		createInstance(Components.interfaces.nsIFileProtocolHandler).
		getFileFromURLSpec(url);
	},

	getFilePicker: function(type,fname) {
		var mode;
		var desc = this.getStr('cookiesafe.ImpExpLocation');
		var nsIFilePicker = Components.interfaces.nsIFilePicker;
		var fp = Components.classes['@mozilla.org/filepicker;1'].
        		createInstance(nsIFilePicker);
		if (type == 'save') {
			mode = nsIFilePicker.modeSave;
		} else {
			mode = nsIFilePicker.modeOpen;
		}
		fp.init(window, desc, mode);
		fp.appendFilters(nsIFilePicker.filterAll);
		fp.defaultExtension = '.xml';
		fp.defaultString = fname;
		fp.appendFilters(nsIFilePicker.filterXML);
		fp.appendFilters(nsIFilePicker.filterText);
		var res = fp.show();
		if (res==nsIFilePicker.returnOK || (type == 'save' && res==nsIFilePicker.returnReplace)) {
  			var file = fp.file;
			if (type == 'save' && !file.exists()) {
				file.create(0,null);
			}
			return file;
		}
		return false;
	},

	exportCookies: function() {
		var prefs = this.getPrefs();
		var ask = prefs.getBoolPref('promptExport');
		var cfrm = (ask) ? confirm(this.getStr('cookiesafe.ConfirmOverwriteCk')) : true;
		if (!cfrm) return false;

		var file,ck,host,path,name,value,secure,http,session,expires,escaped;
		var prloc = prefs.getBoolPref('promptImpExpLocation');
		var url = prefs.getCharPref('cookiesLocation');

		if (url) var uri = this.getURI(url);

		if (!url || prloc || uri.scheme != 'file') {
			file = this.getFilePicker('save','cookies.xml');
		} else {
			file = this.convertUrlToFile(uri.spec);
		}

		if (!file || !file.exists()) return false;

		//output the xml header
		var output = '<?xml version=\"1.0\"?>\r\n\r\n<cookies>\r\n';

		try {
			//this will overwrite all existing content
			this.writeFile(file, output, 0x04 | 0x08 | 0x20);
		} catch(e) {
			alert(e);
		}

		//loop through cookies and export them one at a time
		var mngr = this.getCookieManager();
		var cookies = mngr.enumerator;
		while (cookies.hasMoreElements()) {
			ck = cookies.getNext();
			ck.QueryInterface(Components.interfaces.nsICookie);
			ck.QueryInterface(Components.interfaces.nsICookie2);

			escaped = (escape(ck.host) != ck.host) ? true : false; //if true it needs to be escaped
			if (unescape(ck.host) != ck.host) escaped = false; //already escaped so do nothing
			host = (escaped) ? '<host escaped=\'true\'>'+escape(ck.host)+'</host>' : '<host>'+ck.host+'</host>';

			escaped = (escape(ck.path) != ck.path) ? true : false;
			if (unescape(ck.path) != ck.path) escaped = false;
			path = (escaped) ? '<path escaped=\'true\'>'+escape(ck.path)+'</path>' : '<path>'+ck.path+'</path>';

			escaped = (escape(ck.name) != ck.name) ? true : false;
			if (unescape(ck.name) != ck.name) escaped = false;
			name = (escaped) ? '<name escaped=\'true\'>'+escape(ck.name)+'</name>' : '<name>'+ck.name+'</name>';

			escaped = (escape(ck.value) != ck.value) ? true : false;
			if (unescape(ck.value) != ck.value) escaped = false;
			value = (escaped) ? '<value escaped=\'true\'>'+escape(ck.value)+'</value>' : '<value>'+ck.value+'</value>';

			secure = '<secure>'+ck.isSecure+'</secure>';

			http = ('isHttpOnly' in ck) ? '<http>'+ck.isHttpOnly+'</http>' : '<http>false</http>';

			session = '<session>'+ck.isSession+'</session>';

			expires = '<expires>'+ck.expiry+'</expires>';

			output = '\t<cookie>\r\n\t\t'+host+'\r\n\t\t'+path+'\r\n\t\t'+name+'\r\n\t\t'+value+'\r\n\t\t'+
				 secure+'\r\n\t\t'+http+'\r\n\t\t'+session+'\r\n\t\t'+expires+'\r\n\t</cookie>\r\n';

			try {
				//this will append to existing content
				this.writeFile(file, output, 0x04 | 0x10);
			} catch(e) {
				alert(e);
			}
		}

		//output the xml footer
		output = '</cookies>\r\n';

		try {
			//this will append to existing content
			this.writeFile(file, output, 0x04 | 0x10);
		} catch(e) {
			alert(e);
		}

		return false;
	},

	exportExceptions: function() {
		var prefs = this.getPrefs();
		var ask = prefs.getBoolPref('promptExport');
		var cfrm = (ask) ? confirm(this.getStr('cookiesafe.ConfirmOverwriteEx')) : true;
		if (!cfrm) return false;

		var file,exc,perms,host,cap,temp;
		var prloc = prefs.getBoolPref('promptImpExpLocation');
		var url = prefs.getCharPref('exceptionsLocation');

		if (url) var uri = this.getURI(url);

		if (!url || prloc || uri.scheme != 'file') {
			file = this.getFilePicker('save','exceptions.xml');
		} else {
			file = this.convertUrlToFile(uri.spec);
		}

		if (!file || !file.exists()) return false;

		//output the xml header
		var output = '<?xml version=\"1.0\"?>\r\n\r\n<permissions>\r\n';

		try {
			//this will overwrite all existing content
			this.writeFile(file, output, 0x04 | 0x08 | 0x20);
		} catch(e) {
			alert(e);
		}

		//loop through exceptions and export them one at a time
		var mngr = this.getPermManager();
		if (mngr instanceof Components.interfaces.nsIPermissionManager) {
			perms = mngr.enumerator;
			while (perms.hasMoreElements()) {
				exc = perms.getNext();
				exc.QueryInterface(Components.interfaces.nsIPermission);
				if (exc.type=='cookie') {
					output = this.formatExceptionsOutput(exc.host,exc.capability);

					try {
						//this will append to existing content
						this.writeFile(file, output, 0x04 | 0x10);
					} catch(e) {
						alert(e);
					}
				}
			}
		} else { //for TB2 only
			perms = mngr.getAllPermissions().split(' ');
			for (var i=0; i<perms.length; ++i) {
				if (!perms[i]) continue;
				temp = perms[i].split('|');
				output = this.formatExceptionsOutput(temp[0],temp[1]);

				try {
					//this will append to existing content
					this.writeFile(file, output, 0x04 | 0x10);
				} catch(e) {
					alert(e);
				}
			}
		}

		//output the xml footer
		output = '</permissions>\r\n';

		try {
			//this will append to existing content
			this.writeFile(file, output, 0x04 | 0x10);
		} catch(e) {
			alert(e);
		}

		return false;
	},

	writeFile: function(sFilePath, sFileContent, flags) {
		var strm = Components.classes["@mozilla.org/network/file-output-stream;1"].
			createInstance(Components.interfaces.nsIFileOutputStream);
		strm.QueryInterface(Components.interfaces.nsIOutputStream);
		strm.QueryInterface(Components.interfaces.nsISeekableStream);
		strm.init(sFilePath, flags, 420, 0);
		strm.write(sFileContent, sFileContent.length);
		strm.flush();
		strm.close();
	},

	formatExceptionsOutput: function(hst,capability) {
		var escaped = (escape(hst) != hst) ? true : false; //if true it needs to be escaped
		if (unescape(hst) != hst) escaped = false; //already escaped so do nothing
		var host = (escaped) ? '<host escaped=\'true\'>'+escape(hst)+'</host>' : '<host>'+hst+'</host>';

		var cap = '<capability>'+capability+'</capability>';

		var output = '\t<exception>\r\n\t\t'+host+'\r\n\t\t'+cap+'\r\n\t</exception>\r\n';
		return output;
	},

	viewExportedCookies: function() {
		var prefs = this.getPrefs();
		var url = prefs.getCharPref('cookiesLocation');
		if (!url) return false;

		cookiesafeExternal.openInExternalEditor(url,null,null);
		return false;
	},

	viewExportedExceptions: function() {
		var prefs = this.getPrefs();
		var url = prefs.getCharPref('exceptionsLocation');
		if (!url) return false;

		cookiesafeExternal.openInExternalEditor(url,null,null);
		return false;
	},

	clearDomainCookies: function(basepg) {
		var ck,baseck;
		var prefs = this.getPrefs();
		var ask = prefs.getBoolPref('promptClear');
		var clr = (ask) ? confirm(this.getStr('cookiesafe.ConfirmClearDomainCk')+'\n\n'+basepg) : true;
		if (!clr) return false;

		//loop through cookies and clear them one at a time
		var mngr = this.getCookieManager();
		var cookies = mngr.enumerator;
		while (cookies.hasMoreElements()) {
			ck = cookies.getNext();
			ck.QueryInterface(Components.interfaces.nsICookie);
			ck.QueryInterface(Components.interfaces.nsICookie2);

			//make sure cookie relates to the current page
			baseck = this.getBaseDomain(ck.rawHost);
			if (baseck == basepg) {
				mngr.remove(ck.host,ck.name,ck.path,false);
			}
		}
		return false;
	},

	exportDomainCookies: function(basepg) {
		var file,ck,host,path,name,value,secure,http,session,expires,escaped,baseck;
		var prefs = this.getPrefs();
		var prloc = prefs.getBoolPref('promptImpExpLocation');
		var url = prefs.getCharPref('cookiesLocation');

		if (url) var uri = this.getURI(url);

		if (!url || prloc || uri.scheme != 'file') {
			file = this.getFilePicker('save','cookies.xml');
		} else {
			file = this.convertUrlToFile(uri.spec);
		}

		if (!file) return false;

		//output the header
		var output = '<cookies>\r\n';

		try {
			//this will append to existing content
			this.writeFile(file, output, 0x04 | 0x10);
		} catch(e) {
			alert(e);
		}

		//loop through cookies and export them one at a time
		var mngr = this.getCookieManager();
		var cookies = mngr.enumerator;
		while (cookies.hasMoreElements()) {
			ck = cookies.getNext();
			ck.QueryInterface(Components.interfaces.nsICookie);
			ck.QueryInterface(Components.interfaces.nsICookie2);

			//make sure cookie relates to the current page
			baseck = this.getBaseDomain(ck.rawHost);
			if (baseck != basepg) continue;

			escaped = (escape(ck.host) != ck.host) ? true : false; //if true it needs to be escaped
			if (unescape(ck.host) != ck.host) escaped = false; //already escaped so do nothing
			host = (escaped) ? '<host escaped=\'true\'>'+escape(ck.host)+'</host>' : '<host>'+ck.host+'</host>';

			escaped = (escape(ck.path) != ck.path) ? true : false;
			if (unescape(ck.path) != ck.path) escaped = false;
			path = (escaped) ? '<path escaped=\'true\'>'+escape(ck.path)+'</path>' : '<path>'+ck.path+'</path>';

			escaped = (escape(ck.name) != ck.name) ? true : false;
			if (unescape(ck.name) != ck.name) escaped = false;
			name = (escaped) ? '<name escaped=\'true\'>'+escape(ck.name)+'</name>' : '<name>'+ck.name+'</name>';

			escaped = (escape(ck.value) != ck.value) ? true : false;
			if (unescape(ck.value) != ck.value) escaped = false;
			value = (escaped) ? '<value escaped=\'true\'>'+escape(ck.value)+'</value>' : '<value>'+ck.value+'</value>';

			secure = '<secure>'+ck.isSecure+'</secure>';

			http = ('isHttpOnly' in ck) ? '<http>'+ck.isHttpOnly+'</http>' : '<http>false</http>';

			session = '<session>'+ck.isSession+'</session>';

			expires = '<expires>'+ck.expiry+'</expires>';

			output = '\t<cookie>\r\n\t\t'+host+'\r\n\t\t'+path+'\r\n\t\t'+name+'\r\n\t\t'+value+'\r\n\t\t'+
				 secure+'\r\n\t\t'+http+'\r\n\t\t'+session+'\r\n\t\t'+expires+'\r\n\t</cookie>\r\n';

			try {
				//this will append to existing content
				this.writeFile(file, output, 0x04 | 0x10);
			} catch(e) {
				alert(e);
			}
		}

		//output the xml footer
		output = '</cookies>\r\n';

		try {
			//this will append to existing content
			this.writeFile(file, output, 0x04 | 0x10);
		} catch(e) {
			alert(e);
		}

		return false;
	},

	viewCookies: function() {
		var prefs = this.getPrefs();
		var brows = this.getAppInfo();
		if (brows.name=='Thunderbird' || !prefs.getBoolPref('useBrowserDialogs')) {
			window.openDialog("chrome://cookiesafe/content/cookiesafeCookies.xul","_blank","chrome,resizable=yes", null);
			return false;
		}

		if (brows.name=='SeaMonkey' || brows.name=='Mozilla') {
			window.openDialog("chrome://communicator/content/permissions/cookieViewer.xul","_blank", "chrome,resizable=yes", null);
		} else if (brows.name=='Netscape') {
			window.openDialog("chrome://browser/content/cookieviewer/CookieViewer.xul","_blank", "chrome,resizable=yes", "cookieManager");
		} else {
			window.openDialog("chrome://browser/content/preferences/cookies.xul","_blank","chrome,resizable=yes", null);
		}
		return false;
	},

	viewExceptions: function() {
		var introtext,title,params;
		var prefs = this.getPrefs();
		var brows = this.getAppInfo();
		if (brows.name=='Thunderbird' || !prefs.getBoolPref('useBrowserDialogs')) {
			introtext = this.getStr('cookiesafe.ExcWinIntro');
			title = this.getStr('cookiesafe.CookiesafeExceptions');

			params = {  blockVisible   : true, 
					sessionVisible : true, 
					allowVisible   : true, 
					prefilledHost  : "", 
					permissionType : "cookie",
					windowTitle    : title,
					introText      : introtext };

			window.openDialog("chrome://cookiesafe/content/cookiesafePermissions.xul","_blank","chrome,resizable=yes", params);
			return false;
		}

		if (brows.name=='SeaMonkey' || brows.name=='Mozilla') {
			window.openDialog("chrome://communicator/content/permissions/cookieViewer.xul","_blank", "chrome,resizable=yes", null);
			window.setTimeout(changeTabSM,200);
		} else if (brows.name=='Netscape') {
			params = {  blockVisible   : true,
                   		sessionVisible : true,
                   		allowVisible   : true,
                   		prefilledHost  : "",
                   		permissionType : "cookie" };

    			window.openDialog("chrome://browser/content/cookieviewer/CookieExceptions.xul","_blank", "chrome,resizable=yes", params);
		} else {
			introtext = this.getStr('cookiesafe.ExcWinIntro');
			title = this.getStr('cookiesafe.ExcWinTitle');

			params = {  blockVisible   : true, 
					sessionVisible : true, 
					allowVisible   : true, 
					prefilledHost  : "", 
					permissionType : "cookie",
					windowTitle    : title,
					introText      : introtext };

			window.openDialog("chrome://browser/content/preferences/permissions.xul","_blank","chrome,resizable=yes", params);
		}
		return false;
	},

	disableCS: function() {
		var prefs = this.getPrefs();
		var ask = prefs.getBoolPref('promptUninstall');
		var cfrm = (ask) ? confirm(this.getStr('cookiesafe.ConfirmDisable')) : true;
		if (!cfrm) return false;

		var mngr = this.getExtManager();
		if (mngr) {
			var ext = mngr.getItemForID('{9D23D0AA-D8F5-11DA-B3FC-0928ABF316DD}');
			if (ext && ext.name) {
				mngr.disableItem('{9D23D0AA-D8F5-11DA-B3FC-0928ABF316DD}');
				this.restartBrowser();
			}
		}
		return false;
	},

	uninstallCS: function() {
		var prefs = this.getPrefs();
		var ask = prefs.getBoolPref('promptUninstall');
		var cfrm = (ask) ? confirm(this.getStr('cookiesafe.ConfirmUninstall')) : true;
		if (!cfrm) return false;

		var mngr = this.getExtManager();
		if (mngr) {
			var ext = mngr.getItemForID('{9D23D0AA-D8F5-11DA-B3FC-0928ABF316DD}');
			if (ext && ext.name) {
				mngr.uninstallItem('{9D23D0AA-D8F5-11DA-B3FC-0928ABF316DD}');
				this.restartBrowser();
			}
		}
		return false;
	},

	viewAbout: function() {
		window.openDialog("chrome://cookiesafe/content/cookiesafeAbout.xul","_blank", "chrome,centerscreen,resizable=yes", null);
	},

	viewOptions: function() {
		window.openDialog("chrome://cookiesafe/content/cookiesafeOptions.xul","_blank", "chrome,centerscreen,resizable=yes", null);
	},

	viewHelp: function() {
		var brwsr = getBrowser();
		if ('addTab' in brwsr) {
			var tab = brwsr.addTab('http://forum.softwareblaze.com');
			brwsr.selectedTab = tab;
		} else {
			window.content.location = 'http://forum.softwareblaze.com';
		}
	},

	viewWiki: function() {
		var brwsr = getBrowser();
		if ('addTab' in brwsr) {
			var tab = brwsr.addTab('http://cookies.softwareblaze.com');
			brwsr.selectedTab = tab;
		} else {
			window.content.location = 'http://cookies.softwareblaze.com';
		}
	},

	showSupportThread: function(ev,menu,thread) {
		if (ev.which <= 1) return false;

		ev.preventDefault();
		menu.hidePopup();
		var brwsr = getBrowser();
		if ('addTab' in brwsr) {
			var tab = brwsr.addTab('http://forum.softwareblaze.com/viewtopic.php?t='+thread);
			brwsr.selectedTab = tab;
		} else {
			window.content.location = 'http://forum.softwareblaze.com/viewtopic.php?t='+thread;
		}

		return false;
	},

	viewLast10Hosts: function() {
		window.openDialog("chrome://cookiesafe/content/cookiesafeLast10Hosts.xul", "_blank", "chrome,centerscreen,resizable=yes", null);
	},

	viewEditCookies: function() {
		window.openDialog("chrome://cookiesafe/content/cookiesafeEditCookies.xul", "_blank", "chrome,centerscreen,resizable=yes", null);
	},

	globalPrompt: function() {
		var prefs = this.getPrefs();
		var bhvr = prefs.getIntPref('globalBehavior');
		if (bhvr == 2) {
			window.openDialog("chrome://cookiesafe/content/cookiesafeGlobalPrompt.xul", "_blank", "chrome,centerscreen,resizable=yes", null);
		} else {
			this.allowAll(true,bhvr);
		}
	},

	cleanCookies: function() {
		window.openDialog("chrome://cookiesafe/content/cookiesafeCleanCookies.xul", "_blank", "chrome,centerscreen,resizable=yes", null);
	}
};


var cookiesafeStartup = {

	init: function() {
		//check if browser is TB2
		var brows = cookiesafe.getAppInfo();
		var num = parseInt(brows.version);
		if (brows.name=='Thunderbird' && num==2) {
			cookiesafe.TB2 = true;
		}

		//uninstall cookiesafe if found
		var mngr = cookiesafe.getExtManager();
		if (mngr) {
			var ext = mngr.getItemForID('{00084897-021a-4361-8423-083407a033e0}');
			if (ext && ext.name) {
				mngr.uninstallItem('{00084897-021a-4361-8423-083407a033e0}');
				cookiesafe.restartBrowser();
			}
		}

		//check if cshostperm.sqlite db exists and is writable for TB2
		if (cookiesafe.TB2) {
			var permdb = cookiesafe.getProfile();
			permdb.append('cshostperm.sqlite');
			if (!permdb.exists()) {
				var permMngr = cookiesafe.getPermManager();
				permMngr.copyCSDBToProfile();
				if (!permdb.exists()) {
					cookiesafe.showDBWarning('notfound',permdb.path);
					return false;
				}
			}
			if (!permdb.isWritable()) {
				cookiesafe.showDBWarning('readonly',permdb.path);
				return false;
			}
		}

		//clear temp exceptions here in case browser crashed before
		//the task was completed on shutdown
		cookiesafe.removeTempExceptions();

		//clear last 10 hosts array
		var lt = cookiesafe.getCSLast10Hosts();
		lt.clearLastTenHosts();

		//deny cookies globally based on user pref
		var prefs = cookiesafe.getPrefs();
		var dglobal = prefs.getBoolPref('denyOnStartup');
		if (dglobal) {
			var gPrefs = cookiesafe.getGlobalPrefs();
			gPrefs.setIntPref('cookieBehavior',2);
		}

		//check for updates
		/*var auto = prefs.getIntPref('autoUpdate');
		if (auto) {
			var dt = new Date();
			var time = parseInt(dt.getTime()/1000);
			var last = parseInt(prefs.getIntPref('lastUpdate'));
			if ((auto==1 && time>(last+604800)) ||
			    (auto==2 && time>(last+2629744))) {
				cookiesafeUpdate.updateBlocklist();
			}
		}*/

		//import cookies based on user pref
		var impck = prefs.getBoolPref('importCookiesLoad');
		if (impck) cookiesafe.importCookies(true);

		//import exceptions based on user pref
		var impex = prefs.getBoolPref('importExceptionsLoad');
		if (impex) cookiesafe.importExceptions(true);
		return false;
	}
};
